<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00001000ItemFormPrecoPropostaFactory
{
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        
        $this->connection = $entityManager->getConnection();
    }
    
     /**
      *  A função "insereItemContFormPreco" insere os valores da formação de preço
      *  no item do contrato.
     */
    
    public function insereItemContFormPreco($params){
        
        $sql = "INSERT INTO FCT_ITEMORCAMENTO (
                    NRITEMORCAMENTO,   NRSEQORCAMENTO,     CDSERVORCA,       NMITEMORCAMENTO,    QTITEMORCAMENTO,    CDTPMEDIDASERV,
                    QTMEDIDAORCA,      VRUNITITEMORCA,     VRTOTALITEMORCA,  IDMULTMEDVRTOT,     IDITEMOBRIGORCA,    IDSOMASUBITENS,
                    IDSOMANOITEMPAI,   IDSOMANOTOTALORCA,  IDIMPRESSAOORCA,  IDPRECOLOCACAO,     IDBLOQALTORC,       IDADICITEMORC,
                    IDADICCOBRITEMORC, IDTPDESCORCA,       NRORG,            IDATIVO,            DTINCLUSAO,         NRORGINCLUSAO,
                    CDOPERINCLUSAO,    VRITEMSEMDESC,      NRTIPOORCAMENTO,  NRITEMORCAMENTOPAI, CDTPMEDIDAVARIAVEL
                ) VALUES (
                    :NRITEMORCAMENTO,                 :NRSEQORCAMENTO,            :CDSERVORCA,                  :NMITEMORCAMENTO,          ROUND(:QTITEMORCAMENTO, 2), :CDTPMEDIDASERV,
                    ROUND(NVL(:QTMEDIDAORCA, 1), 2),  ROUND(:VRUNITITEMORCA, 2),  ROUND(:VRTOTALITEMORCA, 2),   NVL(:IDMULTMEDVRTOT,'N'),  'N',                        'N', 
                    'S',                              'S',                        'S',                          'PROPORCIONAL_TEMPO',      'N',                        'N',
                    'N',                              'N',                        :NRORG,                       'S',                       SYSDATE,                    :NRORG,
                    :CDOPERADOR,                      ROUND(:VRTOTALITEMORCA, 2), :NRTIPOORCAMENTO,             :NRITEMORCAMENTOPAI,       :CDTPMEDIDAVARIAVEL
                )";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * A função "somaContFormPreco" edita os itens do contrato que possuem serviços 
     * repetidos na formação dew preço.
     */
    public function somaContFormPreco($params, $medida){
        $sqlAUX = ($medida == 'S') ? " QTMEDIDAORCA = '1', " : '';
        
        $sql = "UPDATE FCT_ITEMORCAMENTO
                   SET VRUNITITEMORCA  = ROUND(VRTOTALITEMORCA + :VRTOTALITEMORCA,2),
                       QTITEMORCAMENTO = '1', 
                       ". $sqlAUX ."
                       VRTOTALITEMORCA = ROUND(VRTOTALITEMORCA + :VRTOTALITEMORCA,2)
                 WHERE CDSERVORCA     = :CDSERVORCA
                   AND NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * A função "retornaItensInseridos" retorna os itens de formação de preço que foram inseridos no contrato
     */
    public function retornaItensInseridos ($codigos, $proposta ,$nrorg) {
        $sql = "SELECT NRITEMORCAMENTO, NRSEQORCAMENTO, CDSERVORCA, NMITEMORCAMENTO, VRTOTALITEMORCA
                  FROM FCT_ITEMORCAMENTO
                 WHERE NRITEMORCAMENTO  IN (".$codigos.")
                   AND NRSEQORCAMENTO    = :NRSEQORCAMENTO
                   AND NRORG            = :NRORG";
        
        $params = array("NRSEQORCAMENTO" => $proposta,
                        "NRORG" => $nrorg);
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return  $std->fetchAll();
    }
    
    /**
     *  A função "insereRateio" insere os rateios dos itens do contrato
     */
    public function insereRateio(array $params) {
        $sql = "INSERT INTO FCT_RELACESTRUTURAPROP (
                    NRRELACESTRUTPROP,  NRSEQORCAMENTO, NRITEMORCAMENTO, NRESTRUTURAM,
                    VRITEMRATEIO,       NRORG,          IDATIVO,         DTINCLUSAO,
                    NRORGINCLUSAO,      CDOPERINCLUSAO, VRPERCRATEIO,    IDRELACESTRUTURA
                ) VALUES (
                    :NRRELACESTRUTPROP, :NRSEQORCAMENTO, :NRITEMORCAMENTO,        :NRESTRUTURAM,
                    :VRITEMRATEIO,      :NRORG,          :IDATIVO,                SYSDATE,
                    :NRORGINCLUSAO,     :CDOPERINCLUSAO, ROUND(:VRPERCRATEIO, 2), :IDRELACESTRUTURA
                )";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     *  A função "atualizaRateio" atualiza os rateios dos itens do contrato
     */
    public function atualizaRateio(array $params) {
        $sql = " UPDATE FCT_RELACESTRUTURAPROP
                    SET VRITEMRATEIO     = ROUND(VRITEMRATEIO + :VRITEMRATEIO, 2)
                  WHERE NRSEQORCAMENTO   = :NRSEQORCAMENTO  
                    AND NRITEMORCAMENTO  = :NRITEMORCAMENTO   
                    AND NRESTRUTURAM     = :NRESTRUTURAM
                    AND NRORG            = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     *  atualiza o tipo de estrutura de um registro da tabela 
     */
    public function atualizarTpEstruturaRateiosInseridos(array $params) {
        $sql = "UPDATE FCT_RELACESTRUTURAPROP
                    SET IDRELACESTRUTURA = :IDRELACESTRUTURA
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO  
                    AND NRORG        = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

     /**
      *  A função "atualizaRateio" valida se ja existe um rateio criado 
      *  para o mesmo item e estrutura
     */
    public function buscaRateio ($params) {
        $sql = "SELECT COUNT (*) AS EXISTE
                 FROM FCT_RELACESTRUTURAPROP
                WHERE NRSEQORCAMENTO   = :NRSEQORCAMENTO  
                  AND NRITEMORCAMENTO  = :NRITEMORCAMENTO   
                  AND NRESTRUTURAM     = :NRESTRUTURAM
                  AND NRORG            = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return  $std->fetchAll();
    }
    
    public function inserirRelacionamentoFormacaoPrecoProposta(array $params) {
        $sql = "INSERT INTO FCT_RELACFORMACAOPROP (
                    NRRELACFORMACAOPROP, NRFORMPRECOSERVICO, NRSEQORCAMENTO, NRITEMORCAMENTO,
                    IDATIVO,             NRORG,              NRORGINCLUSAO,  DTINCLUSAO,
                    CDOPERINCLUSAO
                ) VALUES (
                    :NRRELACFORMACAOPROP, :NRFORMPRECOSERVICO, :NRSEQORCAMENTO, :NRITEMORCAMENTO,
                    :IDATIVO,             :NRORG,              :NRORG,          SYSDATE,
                    :CDOPERINCLUSAO
                )";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getIdsAlteracoesFormacaoPrec(array $params) {
        $sql = "SELECT NRCALCFORMPRECSERV
                FROM FCT_CALCFORMPRECSERV_ITPRO
                WHERE IDITEMCALCULO IN :IDITEMCALCULO
                    AND NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                    AND NRORG              = :NRORG
                    AND NRSEQORCAMENTO     = :NRSEQORCAMENTO
                    AND NRITEMORCAMENTO  IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Deleta as formções não utilizadas
    **/
    public function deletaAlteracoesInutilizadas(array $params) {
        $sql = "DELETE FROM FCT_CALCFORMPRECSERV_ITPRO
                WHERE (NRSEQORCAMENTO   = :NRSEQORCAMENTO
                    AND NRORG           = :NRORG
                    AND NRITEMORCAMENTO = :NRITEMORCAMENTO)
                OR ((:NRITEMORCAMENTO IS NULL AND NRITEMORCAMENTO IS NULL) OR NRITEMORCAMENTO = -1)
                OR ((:NRSEQORCAMENTO  IS NULL AND NRSEQORCAMENTO IS NULL)  OR NRSEQORCAMENTO = 0)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * edita as formações utilizadas adicionando a elas o número do item gerado
    **/
    public function editaItensFormacaoAux(array $params, $iditemcalculo) {
        $sql = "UPDATE FCT_CALCFORMPRECSERV_ITPRO
                SET NRITEMORCAMENTO       = :NRITEMORCAMENTO
                WHERE IDITEMCALCULO IN('".$iditemcalculo."')
                    AND NRORG              = :NRORG
                    AND NRSEQORCAMENTO     = :NRSEQORCAMENTO
                    AND NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                    AND NRITEMORCAMENTO IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Deleta o item de formação de preço inserido quando a rotina de alteração do número 
     * do item na tabela de formção de preço auxiliar não funciona
    **/
    public function deletarItensFormacaoProp(array $params, $nritemorcamento){
        $sql = "DELETE FROM FCT_ITEMORCAMENTO
                WHERE NRSEQORCAMENTO     = :NRSEQORCAMENTO
                    AND NRITEMORCAMENTO IN(".$nritemorcamento.")
                    AND NRORG           = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    /**
     * deleta a árvore completa de elementos alterados na formação de 
     * preço auxiliar já inserida na proposta
     **/
    public function deletarArvoreFormPrecoItPro(array $params){
        $sql = "DELETE FROM FCT_CALCFORMPRECSERV_ITPRO
                WHERE NRCALCFORMPRECSERV IN(SELECT NRCALCFORMPRECSERV
                                                FROM FCT_CALCFORMPRECSERV_ITPRO
                                                WHERE (NRITEMORCAMENTO  = :NRITEMORCAMENTO OR NRITEMORCAMENTO = -1)
                                                    AND (NRSEQORCAMENTO = :NRSEQORCAMENTO  OR NRSEQORCAMENTO  = 0)
                                                    AND NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                                                    AND NRORG              = :NRORG
                                                CONNECT BY PRIOR NRCALCFORMPRECSERV = NRCALCFORMPRECSERVPAI
                                                START WITH NRCALCFORMPRECSERV = :NRCALCFORMPRECSERV)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Deleta a relação de formação de preço com a proposta 
     * após a exclusao de um item
    **/
    public function deletarRelacFormacaoProp(array $params){
        $sql = "DELETE FROM FCT_RELACFORMACAOPROP
                WHERE NRSEQORCAMENTO    = :NRSEQORCAMENTO
                    AND NRITEMORCAMENTO = :NRITEMORCAMENTO
                    AND NRORG           = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * retorna a contagem de rateios de uma proposta
    **/
    public function getRateiosItensProposta(array $params, $itens) {
        $sql = "SELECT DISTINCT ITEST.NRESTRUTURAM, 
                    LISTAGG(ITEM.NRITEMORCAMENTO, ',') WITHIN GROUP (ORDER BY ITEM.NRITEMORCAMENTO) AS NRITEMORCAMENTO
                FROM FCT_ITEMORCAMENTO ITEM
                LEFT JOIN FCT_RELACESTRUTURAPROP ITEST
                    ON ITEM.NRITEMORCAMENTO = ITEST.NRITEMORCAMENTO
                    AND ITEM.NRORG          = ITEST.NRORG
                WHERE ITEM.NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND ITEM.NRORG        = :NRORG
                    AND ITEM.NRITEMORCAMENTO NOT IN($itens)
                    AND (ITEST.IDRELACESTRUTURA IS NULL OR ITEST.IDRELACESTRUTURA = 'RATEIO')
                GROUP BY ITEST.NRESTRUTURAM";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Busca os dados dos parametros de faturamento do escopo de uma proposta
    **/
    public function getIdGeraRateioParametros($nrtipoorcamento, $nrorg) {
        $sql = "SELECT IDGERARATEIO
                FROM FCT_CAPAPARAMFATESC
                WHERE NRTIPOORCAMENTO IN($nrtipoorcamento)
                    AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Busca a contagem de estruturas de uma proposta
    **/
    public function getTotalEstrututasItensProposta(array $params) {
        $sql = "SELECT DISTINCT COUNT(*) AS COUNTITENSRATEIO
                FROM FCT_RELACESTRUTURAPROP ESTRUTURA
                WHERE ESTRUTURA.NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND ESTRUTURA.NRORG        = :NRORG
                    AND (SELECT COUNT(*)
                        FROM FCT_ITEMORCAMENTO ITEM
                        LEFT JOIN FCT_RELACESTRUTURAPROP EST
                            ON ITEM.NRITEMORCAMENTO = EST.NRITEMORCAMENTO
                            AND ITEM.NRORG          = EST.NRORG
                        WHERE ITEM.NRSEQORCAMENTO = :NRSEQORCAMENTO
                            AND ITEM.NRORG        = :NRORG
                            AND EST.NRESTRUTURAM IS NULL) = 0";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * retorna o valor total do item de uma proposta
    **/
    public function getValorTotalItem(array $params) {
        $sql = "SELECT VRTOTALITEMORCA 
                FROM FCT_ITEMORCAMENTO
                WHERE NRITEMORCAMENTO = :NRITEMORCAMENTO
                  AND NRSEQORCAMENTO  = :NRSEQORCAMENTO
                  AND NRORG           = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * busca as estruturas sem centro de custo
    **/
    public function getEstruturasSemCentroCusto($estruturas) {
        $sql = "SELECT NRESTRUTURAM
                FROM ESTRUTURAM
                WHERE NRESTRUTURAM IN ($estruturas)
                    AND CDCENTCUST IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
}

?>